C ----------------------------------------------------------------------
C
C Fr  Regression lineaire multiple par la methode de Gauss-Jordan
C     avec tests: (1) par permutation des donnees brutes ou (2) par
C     permutation des residus sous le modele complet (ter Braak 1992).
C     Details sur ces methodes: Legendre & Legendre (1998) ainsi que
C     Anderson & Legendre (1999).
C
C     Cette version permet de calculer la regression lineaire par l'origine
C     et de tester les parametres par permutation (Legendre & Desdevises, MS).
C
C Eng Multiple linear regression using the Gauss-Jordan method
C     with permutation tests using: (1) permutation of the raw data
C     or (2) permutation of residuals of a full model (ter Braak 1992).
C     Details about these permutation methods are given in Legendre 
C     & Legendre (1998) as well as Anderson & Legendre (1999).
C
C     This version computes linear regression through the origin and
C     includes two forms of permutation tests (Legendre & Desdevises, MS).
C
C ----------------------------------------------------------------------
C
C     References:
C
C     Anderson, M. J. and P. Legendre. 1999. An empirical comparison of 
C        permutation methods for tests of partial regression 
C        coefficients in a linear model. Journal of Statistical 
C        Computation and Simulation 62: 271-303.
C
C     Legendre, P. and L. Legendre. 1998. Numerical ecology, 2nd English 
C        edition. Elsevier Science BV, Amsterdam.
C
C     Legendre, P. and Y. Desdevises. Independent contrasts and regression
C        through the origin. Manuscript.
C
C     ter Braak, C. J. F. 1992. Permutation versus bootstrap significance 
C        tests in multiple regression and ANOVA. 79-86 in: K.-H. Joeckel,
C        G. Rothe & W. Sendler [eds.] Bootstrapping and related 
C        techniques. Springer-Verlag, Berlin. 
C
C ----------------------------------------------------------------------
C
C     nmax = nombre maximum d'objets (n = nombre reel d'objets)
C     mmax = nombre maximum de variablespredictives 
C            (m = nombre reel de variables X).
C
C     Entree des donnees:
C     Un seul fichier de donnees pour les variables X et y,
C     ou bien des fichiers separes pour les variables X et y.
C
C     Pour le calcul, une colonne de 1 est ajoutee par le programme dans
C     la premiere colonne de la matrice interne X.
C
C     On peut faire ecrire dans un fichier les valeurs ajustees de la 
C     regression.
C     
C                                             Pierre Legendre, 1998-2002
C
C ----------------------------------------------------------------------
C2345678901234567890123456789012345678901234567890123456789012345678901234567890
      Integer n,nmax,m,mmax
      Parameter (nmax=1000,mmax=101)
      Integer iordre(nmax),iGEt(mmax)
      Integer iGauss1(mmax),iGauss2(mmax),iGauss3(mmax)
      Real*8 GaussB(mmax,1)
      Real*8 A(mmax,nmax),b(mmax,1),bper(mmax,1),Inverse(mmax,mmax),
     +       probT(mmax),probTper(mmax),res(nmax),t(mmax),tper(mmax),
     +       X(nmax,mmax),y(nmax,1),yper(nmax,1),Z(mmax,nmax)
C BBEdit is the 'creator' for output files
C      call F_Creator('R*ch')
      idebug=0
C Langue du dialogue / Dialog language: 1 = francais; 2 = English
      open(7,file='Regression.out',status='NEW')
    2 write(*,*) 'Francais: tapez (1)'
      write(*,*) 'English:  type  (2)'
      read(*,*) langue
      if((langue.lt.1).or.(langue.gt.2)) goto 2
      if(langue.eq.1) then
         write(*,106)
         write(7,106)
         else
         write(*,206)
         write(7,206)
         endif
C Informations preliminaires
      call Lecture(n,m,X,y,nmax,mmax,langue,intercpt,idebug)
    4 if(langue.eq.1) write(*,*) 'Coefficients de regression: ',
     +   'Test (1) unilateral ou (2) bilateral?'
      if(langue.eq.2) write(*,*) 
     +'Regression coefficients: (1) one-tailed or (2) two-tailed test?'
      read(*,*) itail
      if((itail.lt.1).or.(itail.gt.2)) goto 4
      write(*,*)
    6 if(m.eq.intercpt+1) then
         if(langue.eq.1) then
            write(*,*) 
     +      'Pour le test des coefficients de regression: '
            write(*,*) '  (0) pas de test par permutation'
            write(*,*) '  (1) permuter les donnees brutes'
            else
            write(*,*) 'To test the regression coefficients: '
            write(*,*) '  (0) no permutation test'
            write(*,*) '  (1) permute the raw data'
            endif
         else
         if(intercpt.eq.0) then
            if(langue.eq.1) then
               write(*,*) 
     +         'Pour le test des coefficients de regression: '
               write(*,*) '  (0) pas de test par permutation'
             write(*,*) '  (1) permuter les donnees brutes (recommande)'
               write(*,*) '  (2) ',
     +         'permuter les residus du modele de regression complet'
               else
               write(*,*) 'To test the regression coefficients: '
               write(*,*) '  (0) no permutation test'
               write(*,*) '  (1) permute the raw data (recommended)'
               write(*,*) '  (2) ',
     +         'permute the residuals of the full regression model'
               endif
            else
            if(langue.eq.1) then
               write(*,*) 
     +         'Pour le test des coefficients de regression: '
               write(*,*) '  (0) pas de test par permutation'
               write(*,*) '  (1) permuter les donnees brutes'
               write(*,*) '  (2) permuter',
     +       ' les residus du modele de regression complet (recommande)'
               else
               write(*,*) 'To test the regression coefficients: '
               write(*,*) '  (0) no permutation test'
               write(*,*) '  (1) permute the raw data'
               write(*,*) '  (2) permute',
     +       ' the residuals of the full regression model (recommended)'
               endif
            endif
         endif
      read(*,*) mode
      write(*,*)
      if(m.eq.intercpt+1) then
         if((mode.lt.0).or.(mode.gt.1)) goto 6
         else
         if((mode.lt.0).or.(mode.gt.2)) goto 6
         endif
      nperm=0
      if(mode.gt.0) then
         if(langue.eq.1) write(*,*) 'Combien de permutations? ',
     +      '(e.g. 999, 9999, ...)'
         if(langue.eq.2) write(*,*) 'How many permutations? ',
     +      '(e.g. 999, 9999, ...)'
         read(*,*) nperm
         write(*,*)
         endif
   12 if(langue.eq.1) then
         write(*,*) 'Voulez-vous les valeurs ajustees ',
     +              'ainsi que les residus?'
         write(*,*) '(0) non,  (1) oui'
         else
         write(*,*) 'Do you want the fitted values and residuals?'
         write(*,*) '(0) no,  (1) yes'
         endif
      read(*,*) ires
      if((ires.lt.0).or.(ires.gt.1)) goto 12
      if(ires.eq.1) then
         if(langue.eq.1) then
            open(8,file='Valeurs ajustees et residus',status='NEW')
            write(*,*) "Ces valeurs se trouveront ",
     +         "dans le fichier 'Valeurs ajustees et residus'"
            write(8,*) ' Val.ajustees       Residus'
            else
            open(8,file='Fitted values and residuals',status='NEW')
            write(*,*) "They will be written out ",
     +         "to the file 'Adjusted values and residuals'"
            write(8,*) '   Fitted val.    Residuals'
            endif
            write(8,*)
         endif
C Demarrer l'horloge
      debut=secnds(0.0)
      write(*,*)
C
      call Regression(n,m,nmax,mmax,iordre,langue,itail,mode,
     +   nperm,ires,iGEt,A,b,bper,Inverse,probT,probTper,res,t,tper,
     +   X,y,yper,Z,intercpt,idebug,iGauss1,iGauss2,iGauss3,GaussB)
C
C Duree du calcul?
      write(*,*)
      write(7,*)
      fin=secnds(debut)
      if(langue.eq.1) then
         write(*,110) fin
         write(7,110) fin
         write(*,*) "Les resultats se trouvent egalement dans ",
     +              "le fichier 'Regression.out'"
         write(*,*) 'Fin du programme.'
         else
         write(*,210) fin
         write(7,210) fin
         write(*,*) "Results are also found in file 'Regression.out'"
         write(*,*) 'End of the program.'
         endif
      close(7)
      close(10)
      stop
  106 format(/' Programme de regression multiple'/
     +       ' avec tests par permutations.'/
     +       ' Option: ordonnee a l''origine forcee a 0'//
     +       ' Pierre Legendre'/
     +       ' Departement de sciences biologiques'/
     +       ' Universite de Montreal'/
     +       '       (c) Pierre Legendre, 1999, 2002'/)
  206 format(/' Program for multiple regression'/
     +       ' with permutation tests.'/
     +       ' Option: intercept forced to 0'//
     +       ' Pierre Legendre'/
     +       ' Departement de sciences biologiques'/
     +       ' Universite de Montreal'/
     +       '       (c) Pierre Legendre, 1999, 2002'/)
  110 format(' Duree du calcul:',f10.2,' sec.'/)
  210 format(' Computation time:',f10.2,' sec.'/)
      end

C Subroutine GaussJ is modified from Numerical Recipes.
C
C The author of the present program is not authorized to distribute the source
C code of Numerical Recipes subroutines. If you want to use this subroutine to
C recompile the present program (in the original or some modified form) or in
C any other program, you should obtain a license from Numerical Recipes. 
C See the WWWeb site <http://world.std.com/~nr/storefront.html>.
C The subroutine is reproduced here to show how the original Numerical Recipes
C subroutine should be modified before incorporating it in the present program.
C
      Subroutine GaussJ(m,mmax,A,ipiv,indxr,indxc,B,iflag)
      Real*8 A(mmax,mmax),B(mmax,1),val,big,pivinv,dum
      Integer*4 ipiv(mmax), indxr(mmax), indxc(mmax)
C2345678901234567890123456789012345678901234567890123456789012345678901234567890
c-----------------------------------------------------------------------
c 
c Linear equation solution by Gauss-Jordan elimination.
c From 'Numerical Recipes' (Press et al., 1986), p. 28-29.
c A is an input matrix of 'm x m' elements, stored in a array of physical
c dimensions 'mmax x mmax'. B is an input matrix of 'm x ncolB' containing the
c 'ncolB' right-hand side vectors, stored in an array of physical dimension 
c 'mmax x 1'.
c [In this implementation, B only contains a column vector of zero on start.]
c On output, A is replaced by its matrix inverse, and B is replaced by
c the corresponding set of solution vectors.
c 
c Note: a singular matrix causes an appropriate "pause" message. 
c 
c The integer arrays IPIV, INDXR and INDXC are used for bookkeeping on the
c pivoting. 'mmax' should be as large as the largest anticipated value of 'm'.
c 
c Modified for GSLIB from Numerical Recipes (1986), Section 2.1, p. 28-29.
c 
C Solves the system: [B(nxm)|identity] = A(nxn).[X(nxm)|invA] where ncolB=1.
c X (stored in B on input) is the initial solution vector of the set of
c linear equations; on output, B is the set of regression coefficients.
c
c Pour realiser une regression lineaire:
c au depart, B est le vecteur des valeurs X'y;
c A est la matrice [X'X] a inverser;
C a la fin du programme, B contient le vecteur des coefficients de regression.
c-----------------------------------------------------------------------
      if (m .lt. 1) stop 'illogical m in Gaussj.'
      ncolB=1
      iflag=0
C     Initialisation de b
      do 2 i=1,m
      do 2 j=1,ncolB
    2 b(i,j)=0.0
C
      do 10 j = 1, m
      ipiv(j) = 0
   10 continue 
C Beginning the main loop (110) over the columns to be reduced
      do 110 i = 1, m
        big = 0.
C Beginning the outer loop (30) of the search for a pivot element
        do 30 j = 1, m
          if (ipiv(j) .ne. 1) then 
            do 20 k = 1, m
              if (ipiv(k) .eq. 0) then 
                if (abs(a(j,k)) .ge. big) then 
                  big = abs(a(j,k))
                  irow = j
                  icol = k
                end if 
              else if (ipiv(k) .gt. 1) then
                pause 'singular matrix'
              end if 
   20       continue 
          end if 
   30   continue 
        ipiv(icol) = ipiv(icol) + 1
C We now have the pivot element, so we interchange rows, if needed, to put
C the pivot element on the diagonal. The columns are not physically
C interchanged, only relabeled: INDX(i), the column of the i-th pivot, is
C the i-th column that is reduced, while INDXR(i) is the row in which that
C pivot element was originally located. If INDXR(i)<>INDXC(i) there is an
C implied column interchange. With this form of bookkeeping, the solution 
C B's will end up in the correct order, and the inverse matrix will be
C scrambled by columns.
        if (irow .ne. icol) then 
          do 40 l = 1, m
            dum = a(irow,l)
            a(irow,l) = a(icol,l)
            a(icol,l) = dum
   40     continue 
          do 50 l = 1, ncolB
            dum = b(irow,l)
            b(irow,l) = b(icol,l)
            b(icol,l) = dum
   50     continue 
        end if 
        indxr(i) = irow
        indxc(i) = icol
C We are now ready to divide the pivot row by the pivot element, 
C located at Irow and Icol
C       if (dabs(a(icol,icol)) .le. 0.00000001) pause 'Singular matrix.'
        if (dabs(a(icol,icol)) .le. 0.00000001) then
           iflag=1
C           write(*,*) 'Singular matrix'
           return
           endif
        pivinv = 1. / a(icol,icol)
        a(icol,icol) = 1.
        do 60 l = 1, m
          a(icol,l) = a(icol,l) * pivinv
   60   continue 
        do 70 l = 1, ncolB
          b(icol,l) = b(icol,l) * pivinv
   70   continue 
C Next we reduce the rows, except for the pivot one, of course
        do 100 ll = 1, m
          if (ll .ne. icol) then 
            dum = a(ll,icol)
            a(ll,icol) = 0.
            do 80 l = 1, m
              a(ll,l) = a(ll,l) - a(icol,l) * dum
   80       continue 
            do 90 l = 1, ncolB
              b(ll,l) = b(ll,l) - b(icol,l) * dum
   90       continue 
          end if 
  100   continue 
  110 continue 
C End of the main loop over columns of the reduction. 
C It only remains to unscramble the solution in view of the column
C interchanges. We do this by interchanging pairs of columns in the
C reverse order that the permutation was built up.
      do 130 l = m, 1, -1
        if (indxr(l) .ne. indxc(l)) then 
          do 120 k = 1, m
            dum = a(k,indxr(l))
            a(k,indxr(l)) = a(k,indxc(l))
            a(k,indxc(l)) = dum
  120     continue 
        end if 
  130 continue 
C
C End of inversion procedure.
      return
      end
      
      Subroutine Lecture(n,m,X,y,nmax,mmax,langue,intercpt,idebug)
      Real*8 X(nmax,mmax),y(nmax,1)
      Character*79 nom,nom2
C2345678901234567890123456789012345678901234567890123456789012345678901234567890
C Lecture des donnees  
    2 if(langue.eq.1) then
         write(*,*)'Fichier(s) de donnees:'
         write(*,*)'(1) Un seul fichier contenant les variables X et y?'
         write(*,*)'(2) Fichiers separes pour les variables ',
     +             'predictives X et la variable reponse y?'
         else
         write(*,*)'Data file(s):'
         write(*,*)'(1) A single file containing variables X and y?'
         write(*,*)'(2) Separate files for X (predictor variables) ',
     +             'and y (response variable)?'
         endif
      read(*,*) nfich
      if((nfich.lt.1).or.(nfich.gt.2)) goto 2
C
      write(*,*)
    3 if(langue.eq.1) then
         write(*,*) '(0) Ordonnee a l''origine forcee a 0'
         write(*,*) '(1) Modele de regression ordinaire',
     +              ' avec une ordonnee a l''origine'
         else
         write(*,*) '(0) Intercept forced to 0'
         write(*,*) '(1) Ordinary regression model with an intercept'
         endif
      read(*,*) intercpt
      if((intercpt.lt.0).or.(intercpt.gt.1)) goto 3
      goto (4,20) nfich
C Un seul fichier de donnees
    4 write(*,*)
      if(langue.eq.1) then
         write(*,*) 'Fichier de donnees:'
         write(*,*) '(1) Les variables X d''abord, la variable ',
     +                  'reponse y ensuite?'
         write(*,*) '(2) La variable reponse y d''abord, les ',
     +                  'variables X ensuite?'
         else
         write(*,*) 'Data file:'
         write(*,*) '(1) Predictors X first, followed by response ',
     +                  'variable y?'
         write(*,*) '(2) Response variable y first, followed by ',
     +                  'predictors X?'
         endif
      read(*,*) iordre2
      if((iordre2.lt.1).or.(iordre2.gt.2)) goto 4
C
      write(*,*)
      if(langue.eq.1) write(*,*) 'Nom du fichier de donnees?'
      if(langue.eq.2) write(*,*) 'Name of the data file?'
      read(*,*) nom
      open(10,file=nom,status='OLD')
      inquire(unit=10,name=nom)
      if(langue.eq.1) then
         write(*,*) 'Fichier de donnees: ', nom
         write(*,*)
         write(*,*) 'Combien d''objets et de variables predictives?'
         write(*,*) '(Ne pas compter la variable reponse y)'
         else
         write(*,*) 'Input data file: ', nom
         write(*,*)
         write(*,*) 'How many objects and predictor variables?'
         write(*,*) '(Do not count the response variable y)'
         endif
      read(*,*) n,m
      if((n.gt.nmax/2).or.(m.gt.nmax-1)) goto 99
C
      if(langue.eq.1) then
         write(*,101) nom,n,1,m
         write(7,101) nom,n,1,m
         else
         write(*,201) nom,n,1,m
         write(7,201) nom,n,1,m
         endif
C
      if(intercpt.eq.1) then
         m=m+1
         do 10 i=1,n
         X(i,1)=1.0
         goto (6,8) iordre2
    6    read(10,*) (X(i,j), j=2,m),y(i,1)
         goto 10
    8    read(10,*) y(i,1),(X(i,j), j=2,m)
   10    continue
        else
         do 16 i=1,n
         goto (12,14) iordre2
   12    read(10,*) (X(i,j), j=1,m),y(i,1)
         goto 16
   14    read(10,*) y(i,1),(X(i,j), j=1,m)
   16    continue
        endif
      close(10)
      return
C Deux fichiers de donnees
   20 write(*,*)
      if(langue.eq.1) then
         write(*,*) 'Fichier contenant les variables predictives (X)?'
         else
         write(*,*) 'Input file for predictor variables (X)?'
         endif
      read(*,*) nom
      open(10,file=nom,status='OLD')
      inquire(unit=10,name=nom)
      if(langue.eq.1) then
         write(*,*) 'Fichier des variables X: ', nom
         write(*,*)
         write(*,*) 'Combien d''objets et de variables predictives?'
         else
         write(*,*) 'Input file for variables X: ', nom
         write(*,*)
         write(*,*) 'How many objects and predictor variables?'
         endif
      read(*,*) n,m
      if((n.gt.nmax).or.(m.gt.nmax)) goto 99
      if(intercpt.eq.1) then
         m=m+1
         do 22 i=1,n
         X(i,1)=1.0
   22    read(10,*) (X(i,j), j=2,m)
         else
         do 24 i=1,n
   24    read(10,*) (X(i,j), j=1,m)
         endif
      close(10)
C
      write(*,*)
      if(langue.eq.1) then
         write(*,*) 'Fichier contenant la variable reponse (y)?'
         else
         write(*,*) 'Input file for response variable (y)?'
         endif
      read(*,*) nom
      open(10,file=nom,status='OLD')
      inquire(unit=10,name=nom2)
      if(langue.eq.1)write(*,*) 'Fichier contenant la variable y: ',nom2
      if(langue.eq.2)write(*,*) 'Input file for variable y: ', nom2
      do 30 i=1,n
   30 read(10,*) y(i,1)
      close(10)
C
      if(intercpt.eq.1) then
      if(langue.eq.1) then
         write(*,102) nom,nom2,n,1,(m-1)
         write(7,102) nom,nom2,n,1,(m-1)
         else
         write(*,202) nom,nom2,n,1,(m-1)
         write(7,202) nom,nom2,n,1,(m-1)
         endif
        else
      if(langue.eq.1) then
         write(*,102) nom,nom2,n,1,m
         write(7,102) nom,nom2,n,1,m
         else
         write(*,202) nom,nom2,n,1,m
         write(7,202) nom,nom2,n,1,m
         endif
        endif
      return
C
   99 continue
      if(langue.eq.1) write(6,103)
      if(langue.eq.2) write(6,203)
      stop 3
  101 format(/' Fichier de donnees: ',a32/
     +        i6,' objets'/
     +        i6,' variable reponse'/
     +        i6,' variables predictives'/)
  102 format(/' Fichier de donnees pour X: ',a32/
     +        ' Fichier de donnees pour y: ',a32/
     +        i6,' objets'/
     +        i6,' variable reponse'/
     +        i6,' variables predictives'/)
  103 format(' Probleme trop grand: re-dimensionnez le programme.')
  201 format(/' Input data file: ',a32/
     +        i6,' objects'/
     +        i6,' response variable'/
     +        i6,' predictor variables'/)
  202 format(/' Input data file for X: ',a32/
     +        ' Input data file for y: ',a32/
     +        i6,' objects'/
     +        i6,' response variable'/
     +        i6,' predictor variables'/)
  203 format(' Problem too big. Redimension the program.')
      end

      Subroutine Permute (iseed,n,nmax,iordre)
C This subroutine permutes the first 'n' values of vector 'iordre' at random
C in an equiprobable way. This property has been checked through intensive 
C simulations.
      Integer iseed,n,i,itemp,km1,m
      Integer iordre(nmax)
      m=n
      km1=n-1
      do 10 i=1,km1
    8    j = 1 + rand(iseed)*m
         if(j.gt.m) goto 8
         itemp = iordre(m)
         iordre(m) = iordre(j)
         iordre(j) = itemp
         m=m-1
   10 continue
      return
      end

      Function Pof(F,df1,df2)
C ALGORITHM Compute probability of F ratio.
C	Adapted from Collected Algorithms of the CACM
C	Algorithm 322
C	Egon Dorrer
      Integer df1,df2,a,b,i,j
      Real*8  I_PI,pof,F,w, y, z, d, p, F_EPSILON
      F_EPSILON=0.000001
      I_PI=0.3183098861837906715377675	
C
      if((F.lt.F_EPSILON).or.(df1.lt.1).or.(df2.lt.1)) then
         pof=1.0
         return
		 endif
      if(mod(df1,2).gt.0) then
         a=1
         else
         a=2
         endif
      if(mod(df2,2).gt.0) then
         b=1
         else
         b=2
         endif
      w = (F * dfloat(df1)) / dfloat(df2)
      z = 1.0 / (1.0 + w)
	     if (a.eq.1) then
	        if (b.eq.1) then
               p = dsqrt (w)
               y = I_PI
               d = y * z / p
               p = 2.0 * y * datan(p)
            else
               p = dsqrt (w * z)
               d = 0.5 * p * z / w
            endif
	     else if (b.eq.1) then
            p = dsqrt (z)
            d = 0.5 * z * p
            p = 1.0 - p
	     else
            d = z * z
            p = w * z
         endif
	     y = 2.0 * w / z
      do 10 j = (b+2), df2, 2
      d = d * ((1.0 + dfloat(a) / dfloat(j-2)) * z)
      if(a.eq.1) then
         p = p + d * y / dfloat(j-1)
         else
         p = (p + w) * z
         endif
   10 continue
      y = w * z
      z = 2.0 / z
      b = df2 - 2
      do 20 i = (a+2), df1, 2
      j = i + b
      d = d * (y * dfloat(j) / dfloat(i-2))
      p = p - (z * d / dfloat(j))
   20 continue
C Correction for approximation errors suggested in certification 
      if (p.lt.0.0) then
            p = 0.0
         else if (p.gt.1.0) then
            p = 1.0
         endif
      pof=1.0-p
C
      return
      end
      
      Subroutine Produit(Z1,Z2,Z3,n1,n2,n3,n1max,n2max,n3max)
      Real*8 Z1(n1max,n2max),Z2(n2max,n3max),Z3(n1max,n3max),temp
C Z1 * Z2 = Z3
      do 8 i=1,n1
      do 8 j=1,n3
      temp=0.0
      do 6 k=1,n2
    6 temp=temp+Z1(i,k)*Z2(k,j)
    8 Z3(i,j)=temp
      return
      end
     
      Subroutine Regression(n,m,nmax,mmax,iordre,langue,itail,mode,
     +   nperm,ires,iGEt,A,b,bper,Inverse,probT,probTper,res,t,tper,
     +   X,y,yper,Z,intercpt,idebug,iGauss1,iGauss2,iGauss3,GaussB)
      Integer n,nmax,m,mmax
      Integer iordre(nmax),iGEt(mmax),iGEF
      Integer iGauss1(mmax),iGauss2(mmax),iGauss3(mmax)
      Real*8 GaussB(mmax,1)
      Real*8 A(mmax,nmax),b(mmax,1),bper(mmax,1),Inverse(mmax,mmax),
     +       probT(mmax),probTper(mmax),res(nmax),t(mmax),tper(mmax),
     +       X(nmax,mmax),y(nmax,1),yper(nmax,1),Z(mmax,nmax)
      Real*8 dfldf,res2,SEb,sx,sx2,fit,R2,F,probR2,Pof,
     +       R2per,Fper,prperR2,r,rper,dfln,xbar,ybar,temp,
     +       varfit,vartot,varfit2,vartot2,amult1,amult2
      Character sig1*1,sig2*1
C2345678901234567890123456789012345678901234567890123456789012345678901234567890
C Calcul des degres de liberte. m = nombre de parametres estimes.
C En regression normale, m = nombre de variables explicatives + intercept
C En regression par l'origine, m = nombre de variables explicatives
      idf=n-m
      dfldf=dfloat(n-m)
      dfln=dfloat(n)
C Calcul de 'vartot' (sans les d.l.) en vue du calcul de R^2
      sx=0.0
      sx2=0.0
      do 3 i=1,n
      sx=sx+y(i,1)
    3 sx2=sx2+y(i,1)**2
      if(intercpt.eq.0) then
         vartot=sx2
         else
         vartot=sx2-(sx**2/dfln)
         endif
C Former la matrice X'X et trouver son inverse (mxm)
      do 4 i=1,n
      do 4 j=1,m
    4 Z(j,i)=X(i,j)
      call Produit(Z,X,Inverse,m,n,m,mmax,nmax,mmax)
      if(idebug.eq.1) then
         write(*,*) '[X''X] ='
         do 92 i=1,m
   92    write(*,120) (Inverse(i,j), j=1,m)
         endif
      if(m.eq.1) then
         Inverse(1,1)=1.0/Inverse(1,1)
         else
         call GaussJ(m,mmax,Inverse,iGauss1,iGauss2,iGauss3,GaussB,
     +   iflag)
         if(iflag.eq.1) Stop 'Singular matrix'
         endif
      if(idebug.eq.1) then
         write(*,*) 'Inv[X''X] ='
         do 93 i=1,m
   93    write(*,121) (Inverse(i,j), j=1,m)
         endif
  121 Format(10f15.10)
C Former la "residualizing matrix" A = [X'X]-1 X' (mxn)
C puis calculer le vecteur b = A y (mx1)
      call Produit(Inverse,Z,A,m,m,n,mmax,mmax,nmax)
      call Produit(A,y,b,m,n,1,mmax,nmax,1)
C Calculer le vecteur des t (mx1)
      sx=0.0
      sx2=0.0
      res2=0.0
      do 8 i=1,n
      fit=0.0
      do 6 j=1,m
    6 fit=fit+b(j,1)*X(i,j)
      sx=sx+fit
      sx2=sx2+fit**2
      res(i)=y(i,1)-fit
      if(ires.eq.1) write(8,105) fit,res(i)
    8 res2=res2+res(i)**2
      res2=dsqrt(res2/dfldf)
      do 10 j=1,m
      SEb=res2*dsqrt(Inverse(j,j))
   10 t(j)=b(j,1)/SEb
C Calculer la dispersion expliquee par la regression 'varfit' et le R^2
C Note: R^2 = varfit/vartot
C The alternative formula that can produce negative values for R2 in regression
C through origin is not used: R^2 = 1 - (SSE/SSTO) (Neter et al. 1996, p. 163)
      if(intercpt.eq.0) then
         varfit=sx2
         R2=varfit/vartot
         F=R2*dfldf/((1.0-R2)*dfloat(m))
         probR2=Pof(F,m,idf)
         else
         varfit=sx2-(sx**2/dfln)
         R2=varfit/vartot
         F=R2*dfldf/((1.0-R2)*dfloat(m-1))
         probR2=Pof(F,m-1,idf)
         endif

      if(m.eq.intercpt+1) then
         r=dsqrt(R2)
         temp=b(intercpt+1,1)
         r=dsign(r,temp)
         if(langue.eq.1) then
            write(*,112) r
            write(7,112) r
            else
            write(*,212) r
            write(7,212) r
            endif
         endif

      sig2=' '
      if(probR2.le.0.05) sig2='*'
      write(*,103) R2,F,probR2,sig2
      write(7,103) R2,F,probR2,sig2
C Preparation en vue des permutations
      n2=2*n
      do 12 i=1,n
   12 iordre(i)=i
      iseed=0
      do 13 i=1,100
   13 turn=rand(iseed)
      do 14 j=1,m
   14 iGEt(j)=1
      iGEF=1
      istart=intercpt+1
C If(intercpt.eq.0): Form "doubled data set". Recompute "vartot" and "Inverse".
      if(intercpt.eq.0) then
         do i=1,n
            y(n+i,1)= -1*y(i,1)
            do j=1,m
               X(n+i,j)= -1*X(i,j)
               enddo
            enddo
         vartot2=vartot*2.0
C Recompute the "residualizing matrix" A, because it cannot easily be obtained 
C from the results computed for the original matrices of length n.
         do 16 i=1,n2
         do 16 j=1,m
   16    Z(j,i)=X(i,j)
         call Produit(Z,X,Inverse,m,n2,m,mmax,nmax,mmax)
         if(idebug.eq.1) then
            write(*,*) '[X''X] ='
            do 94 i=1,m
   94       write(*,120) (Inverse(i,j), j=1,m)
            endif
         if(m.eq.1) then
            Inverse(1,1)=1.0/Inverse(1,1)
            else
            call GaussJ(m,mmax,Inverse,iGauss1,iGauss2,iGauss3,GaussB,
     +      iflag)
            if(iflag.eq.1) Stop 'Singular matrix'
            endif
         call Produit(Inverse,Z,A,m,m,n2,mmax,mmax,nmax)
         endif
      if(nperm.eq.0) goto 90
      goto (70,80) mode
C
C Debut de la boucle de permutation, mode=1: permutation des donnees brutes - -
C
C if(intercpt.eq.0):
C Permutation en deux etapes pour que les donnees demeurent balancees:
C Etape 1: on permute les n entiers
C Etape 2: on choisit la valeur en position iordre(i) ou [iordre(i)+n]
C 'ipair' prend la valeur 0 ou 1 apres un tirage aleatoire
   70 do 30 k=1,nperm
      call Permute(iseed,n,nmax,iordre)
C
      if(intercpt.eq.0) then
         do 18 i=1,n
         ipos=iordre(i)
         ipair=rand(iseed)*2
         if(ipair.eq.0) then
            amult1= 1.0
            amult2=-1.0
            else
            amult1=-1.0
            amult2= 1.0
            endif
         yper(i,1)=  y(ipos,1)*amult1
         yper(n+i,1)=y(ipos,1)*amult2
   18    continue
         call Produit(A,yper,bper,m,n2,1,mmax,nmax,1)
         sx2=0.0
         res2=0.0
         do i=1,n2
            fit=0.0
            do j=1,m
               fit=fit+bper(j,1)*X(i,j)
               enddo
            sx2=sx2+fit**2
            res2=res2+(yper(i,1)-fit)**2
            enddo
         res2=dsqrt(res2/dfldf)
C
         else
         do i=1,n
            yper(i,1)=y(iordre(i),1)
            enddo
         call Produit(A,yper,bper,m,n,1,mmax,nmax,1)
         sx=0.0
         sx2=0.0
         res2=0.0
         do i=1,n
            fit=0.0
            do j=1,m
               fit=fit+bper(j,1)*X(i,j)
               enddo
            sx=sx+fit
            sx2=sx2+fit**2
            res2=res2+(yper(i,1)-fit)**2
            enddo
         res2=dsqrt(res2/dfldf)
         endif
C
      if(intercpt.eq.0) then
         varfit2=sx2
         R2per=varfit2/vartot2
         Fper=R2per*dfloat(n-m)/((1.0-R2per)*dfloat(m))
         else
         varfit=sx2-(sx**2/dfln)
         R2per=varfit/vartot
         Fper=R2per*dfldf/((1.0-R2per)*dfloat(m-1))
         endif
C
      do 28 j=istart,m
         SEb=res2*dsqrt(Inverse(j,j))
         tper(j)=bper(j,1)/SEb
         goto (24,26) itail
   24       if(t(j).ge.0.0) then
               if(tper(j).ge.t(j)) iGEt(j)=iGEt(j)+1
               else
               if(tper(j).le.t(j)) iGEt(j)=iGEt(j)+1
               endif
            goto 28
   26       if(dabs(tper(j)).ge.dabs(t(j))) iGEt(j)=iGEt(j)+1
   28    continue
C
      if(Fper.ge.F) iGEF=iGEF+1
  122 format('isim =',i3,'  itail =',i2,'  k =',i3,'  t =',f9.5,
     +       '  tper =',f9.5,'  iGEt =',i4,'  iGEF =',i4)
   30 continue
C Fin de la boucle de permutation, mode=1 - - - - - - - - - - - - - - - - - - -
      goto 90
C
C Boucle de permutation pour R2, mode=2: perm. des donnees brutes pour R2 - - -
   80 do 38 k=1,nperm
      call Permute(iseed,n,nmax,iordre)
      if(intercpt.eq.0) then
         do 32 i=1,n
         ipos=iordre(i)
         ipair=rand(iseed)*2
         if(ipair.eq.0) then
            amult1= 1.0
            amult2=-1.0
            else
            amult1=-1.0
            amult2= 1.0
            endif
         yper(i,1)=  y(ipos,1)*amult1
         yper(n+i,1)=y(ipos,1)*amult2
   32    continue
         call Produit(A,yper,bper,m,n2,1,mmax,nmax,1)
         sx2=0.0
         res2=0.0
         do i=1,n2
            fit=0.0
            do j=1,m
               fit=fit+bper(j,1)*X(i,j)
               enddo
            sx2=sx2+fit**2
            enddo
C
         else
         do 34 i=1,n
   34    yper(i,1)=y(iordre(i),1)
         call Produit(A,yper,bper,m,n,1,mmax,nmax,1)
         sx=0.0
         sx2=0.0
         res2=0.0
         do i=1,n
            fit=0.0
            do j=1,m
               fit=fit+bper(j,1)*X(i,j)
               enddo
            sx=sx+fit
            sx2=sx2+fit**2
            enddo
         endif
C
      if(intercpt.eq.0) then
         varfit2=sx2
         R2per=varfit2/vartot2
         Fper=R2per*dfloat(n-m)/((1.0-R2per)*dfloat(m))
         else
         varfit=sx2-(sx**2/dfln)
         R2per=varfit/vartot
         Fper=R2per*dfldf/((1.0-R2per)*dfloat(m-1))
         endif
C
      if(Fper.ge.F) iGEF=iGEF+1
   38 continue
C Fin de la boucle de permutation pour r, mode=2  - - - - - - - - - - - - - - -
C
C Boucle de permutation pour les b, mode=2: permutation des residus pour b  - -
      do 54 k=1,nperm
      call Permute(iseed,n,nmax,iordre)
      if(intercpt.eq.0) then
         do 42 i=1,n
         ipos=iordre(i)
         ipair=rand(iseed)*2
         if(ipair.eq.0) then
            amult1= 1.0
            amult2=-1.0
            else
            amult1=-1.0
            amult2= 1.0
            endif
         yper(i,1)=  res(ipos)*amult1
         yper(n+i,1)=res(ipos)*amult2
   42    continue
         call Produit(A,yper,bper,m,n2,1,mmax,nmax,1)
         res2=0.0
         do i=1,n2
            fit=0.0
            do j=1,m
               fit=fit+bper(j,1)*X(i,j)
               enddo
            res2=res2+(yper(i,1)-fit)**2
            enddo
         res2=dsqrt(res2/dfldf)
         else
C
         do 44 i=1,n
   44    yper(i,1)=res(iordre(i))
         call Produit(A,yper,bper,m,n,1,mmax,nmax,1)
         res2=0.0
         do i=1,n
            fit=0.0
            do j=1,m
               fit=fit+bper(j,1)*X(i,j)
               enddo
            res2=res2+(yper(i,1)-fit)**2
            enddo
         res2=dsqrt(res2/dfldf)
         endif
C
      do 52 j=istart,m
         SEb=res2*dsqrt(Inverse(j,j))
         tper(j)=bper(j,1)/SEb
         goto (48,50) itail
   48       if(t(j).ge.0.0) then
               if(tper(j).ge.t(j)) iGEt(j)=iGEt(j)+1
               else
               if(tper(j).le.t(j)) iGEt(j)=iGEt(j)+1
               endif
            goto 52
   50       if(dabs(tper(j)).ge.dabs(t(j))) iGEt(j)=iGEt(j)+1
   52    continue
      if(idebug.eq.2) write(*,122) bper(1,1),tper(1),bper(2,1),tper(2)
   54 continue
C Fin de la boucle de permutation pour les b, mode=2  - - - - - - - - - - - - -
C
   90 if(nperm.gt.0) then
         prperR2=dfloat(iGEF)/dfloat(nperm+1)
         sig1=' '
         if(prperR2.le.0.05) sig1='*'
         write(*,111) prperR2,sig1
         write(7,111) prperR2,sig1
         if(langue.eq.1) then
            write(*,108) nperm
            write(7,108) nperm
            else
            write(*,208) nperm
            write(7,208) nperm
            endif
         endif
C
      write(*,*)
      write(7,*)
      if(itail.eq.1) then
         if(langue.eq.1) then
            write(*,*) 'Coefficient(s) de regression: ',
     +        'Test unilateral dans la direction du signe'
            write(7,*) 'Coefficient(s) de regression: ',
     +        'Test unilateral dans la direction du signe'
            else
            write(*,*) 'Regression coefficient(s): ',
     +        'One-tailed test in direction of sign'
            write(7,*) 'Regression coefficient(s): ',
     +        'One-tailed test in direction of sign'
            endif
         else
         if(langue.eq.1) then
            write(*,*) 'Coefficient(s) de regression: Test bilateral'
            write(7,*) 'Coefficient(s) de regression: Test bilateral'
            else
            write(*,*) 'Regression coefficient(s): Two-tailed test'
            write(7,*) 'Regression coefficient(s): Two-tailed test'
            endif
         endif
      if(nperm.gt.0) then
         if(langue.eq.1) then
            if(mode.eq.1) write(7,108) nperm
            if(mode.eq.2) write(7,109) nperm
            if(mode.eq.1) write(*,108) nperm
            if(mode.eq.2) write(*,109) nperm
            else
            if(mode.eq.1) write(7,208) nperm
            if(mode.eq.2) write(7,209) nperm
            if(mode.eq.1) write(*,208) nperm
            if(mode.eq.2) write(*,209) nperm
            endif
         endif
      write(*,*)
      write(7,*)
C
C Calcul des probabilites (permutationnelle et parametrique) des t(b)
      do 60 j=1,m
         F=t(j)**2
         probT(j)=Pof(F,1,idf)
         if(itail.eq.1) probT(j)=probT(j)*0.5
         if((nperm.gt.0).and.(j.ge.intercpt+1)) 
     +      probTper(j)=dfloat(iGEt(j))/dfloat(nperm+1)
   60    continue
C     Impression des resultats
      sig2=' '
      if(probT(1).le.0.05) sig2='*'
      write(*,104)
      write(*,102)
      write(*,104)
      if(intercpt.eq.1) write(*,100) b(1,1), t(1), probT(1),sig2
      write(7,104)
      write(7,102)
      write(7,104)
      if(intercpt.eq.1) write(7,100) b(1,1), t(1), probT(1),sig2
      do 62 j=istart,m
      sig2=' '
      if(probT(j).le.0.05) sig2='*'
      if(nperm.eq.0) then
         write(*,106) (j-intercpt),b(j,1), t(j), probT(j), sig2
         write(7,106) (j-intercpt),b(j,1), t(j), probT(j), sig2
         else
         sig1=' '
         if(probTper(j).le.0.05) sig1='*'
         write(*,101) (j-intercpt),b(j,1),t(j),probTper(j),sig1,
     +      probT(j),sig2
         write(7,101) (j-intercpt),b(j,1),t(j),probTper(j),sig1,
     +      probT(j),sig2
         endif
   62 continue
      write(*,104)
      write(7,104)
      if(langue.eq.1) then
         write(*,107)
         write(7,107)
         else
         write(*,207)
         write(7,207)
         endif
      return
  100 format(' Intercept',f15.5,f10.5,12x,f10.5,1x,a1)
  101 format(i6,4x,f15.5,2f10.5,1x,a1,f10.5,1x,a1)
  102 format(' Variable',12x,'b         t       P-perm     P-param ')
  103 format
     +(' R^2 =',f9.5,'    F =',f12.5,'    prob (param.) =',f9.5,1x,a1)
  104 format(' -------------------------------------------------',
     +       '----------')
  105 format(5f14.5)
  106 format(i6,4x,f15.5,f10.5,12x,f10.5,1x,a1)
  107 format(' * l''estimation du parametre est significative',
     +       ' au seuil 0.05')
  207 format(' * the parameter estimate is significant',
     +       ' at the 0.05 level')
  108 format(' Nombre de permutations des donnees brutes:',i6)
  208 format(' Number of permutations of raw data:',i6)
  109 format(' Nombre de permutations des residus:',i6)
  209 format(' Number of permutations of residuals:',i6)
  111 format(38x,'prob (perm.)  =',f9.5,1x,a1)
  112 format(' r   =',f9.5,23x,'Test bilateral:')
  212 format(' r   =',f9.5,23x,'Two-tailed test:')
  120 Format(10f15.5)
      end
